From a786e2cded217b36192dc54ac5f5291f862d0c39 Mon Sep 17 00:00:00 2001
From: Markus Winand <markus.winand@winand.at>
Date: Sun, 8 Jul 2018 18:53:35 +0000
Subject: [PATCH 2/2] XML: avoid xmlStrdup if possible

---
 src/backend/utils/adt/xml.c | 12 ++++++++----
 1 file changed, 8 insertions(+), 4 deletions(-)

diff --git a/src/backend/utils/adt/xml.c b/src/backend/utils/adt/xml.c
index 7cef817..cc88a66 100644
--- a/src/backend/utils/adt/xml.c
+++ b/src/backend/utils/adt/xml.c
@@ -4555,6 +4555,7 @@ XmlTableGetValue(TableFuncScanState *state, int colnum,
 			else if (count == 1)
 			{
 				xmlChar    *str;
+				int        free_str = 0; /* xmlFree(str) required? */
 				xmlNodePtr	node;
 
 				node = xpathobj->nodesetval->nodeTab[0];
@@ -4562,6 +4563,7 @@ XmlTableGetValue(TableFuncScanState *state, int colnum,
 				{
 					/* XML elements need recursive processing */
 					str = xml_xmlnodetostring(xtCxt->doc, node);
+					free_str = 1;
 				}
 				else  if (node->type == XML_CDATA_SECTION_NODE ||
 					node->type == XML_TEXT_NODE ||
@@ -4569,7 +4571,7 @@ XmlTableGetValue(TableFuncScanState *state, int colnum,
 					node->type == XML_COMMENT_NODE)
 				{
 					/* Node types that store their data in "content" */
-					str = xmlStrdup(node->content);
+					str = node->content;
 				}
 				else
 				{
@@ -4577,7 +4579,7 @@ XmlTableGetValue(TableFuncScanState *state, int colnum,
 					 * The remaining types (e.g. attributes)
 					 * store their data in the child.
 					 */
-					str = xmlStrdup(node->xmlChildrenNode->content);
+					str = node->xmlChildrenNode->content;
 				}
 
 				if (str != NULL)
@@ -4588,11 +4590,13 @@ XmlTableGetValue(TableFuncScanState *state, int colnum,
 					}
 					PG_CATCH();
 					{
-						xmlFree(str);
+						if (free_str)
+							xmlFree(str);
 						PG_RE_THROW();
 					}
 					PG_END_TRY();
-					xmlFree(str);
+					if (free_str)
+						xmlFree(str);
 				}
 				else
 				{
-- 
1.9.1

